Attribute VB_Name = "mMain"
' ////////////////////////////////////////////////////////////////////////////////////////////////////

' VBToolbox DEMO Application - Wallpaper-changer/cycler - Setwall.exe
' Copyright (C) M Shaw - 2008 (Freeware)
' Website: http://vbtoolbox.kerys.co.uk
' Absolutely no warranty - See website for terms and conditions of use
' ////////////////////////////////////////////////////////////////////////////////////////////////////
' Version:
' 1.00 05/Jul/2008 First release post-development of essential DLL functions
' 1.01 07/Jul/2008 Added JPEGCheck() header-check function for VBToolbox v1.16
'                  Will NOT protect against deeply-corrupted JPEG images
'                  Added DLL version checks. Enabled lower-case filetype comparison
' 1.02 09/Jul/2008 Added ListFiles() feature from DLL and randomised wallpapers (DLL v1.17)
'                  Tested OK on Windows NT4.0/SP6a, Windows 2000/SP4, Windows XP/SP2
'                  Checks to prevent accidental random repetition of same image, SPACE=next image
'                  Added debugging traces for educational use
' 1.03 14/Jul/2008 Minor bugfix on correcting input path, new DLL version 1.18
' 1.04 18/Jul/2008 Minor updates, event logging on error. Changed ConvertAndSetWallpaper to return
'                  the applied file-size. Display output file-size. Application-hang event in
'                  Form1.Timer1.Timer() now logs to the event log that the app will self-terminate
' 1.05 31/Aug/2009 Recompiled for DLL v1.21 distro
' 1.06 22/Jul/2010 Changed URL
' ////////////////////////////////////////////////////////////////////////////////////////////////////
' Demo program for VBToolbox functionality (MSLIB145.DLL - v1.15+ required)
' Contains one form (Form1) which is never shown but is used for image conversion
' Demonstrates various functions provided by the VBToolbox DLL

' SETWALL /? or SETWALL /H for help screen
' SETWALL <wallpaper-file-path> [options]
' SETWALL flowers.jpg /Q /CYCLE
' SETWALL c:\files\landscape.bmp /INTERVAL:60 /CYCLE
' SETWALL d:\images /Q /TIME:60 /CYCLE ' Get images from d:\images, change every 60 seconds
'
' ////////////////////////////////////////////////////////////////////////////////////////////////////
' IMPORTANT: * This program is not a JPEG validator for damaged images *

' Note: Corrupted JPEG files can cause Visual BASIC and VB to hang when the image is
' loaded into a .Picture object. This can happen even if other applications can handle
' and workaround corrupted file problems. You should verify any JPEG image collections
' for errors if you find SetWall halts or hangs at a given JPEG image. This is a problem
' with the VB control which hangs without reporting an error to the error-handler if it
' encounters a corrupted JPEG image (i.e. premature end of file due to bad-download).
' Download "Bad Peggy" JPEG Validator from http://www.hotpixel.net (Free)
' If in doubt open the suspect image in an graphics editor and re-save to disk
' ////////////////////////////////////////////////////////////////////////////////////////////////////
' Improvements and TODO List:
' Recurse subdirectories - may add DLL feature to do this
' DONE Randomise wallpaper (read all relevant files to a (big) array and select at random
' ////////////////////////////////////////////////////////////////////////////////////////////////////


' Declares ///////////////////////////////////////////////////////////////////////////////////////////

Option Base 0                       ' Arrays are based at 0 (change if required)
Option Explicit                     ' Vars must be declared

Const AUTHOR = "M Shaw"
Const WEBSITE = "http://vbtoolbox.kerys.co.uk"
Const VERDATE = "22nd July 2010"
Const EXENAME = "SetWall"           ' The program name
Const VERSION = "1.06"              ' Setwall (Not DLL) version
Const DLLNAME = "MSLIB145.DLL"      ' Unfortunately, cannot use this in delcares
Const LIBNEEDED As Integer = 118    ' We need this version or higher (n/100)

Public ImageERR As Long             ' Local error-handler for image-conversion
Public ERL As Integer               ' Error-handler trace for main (line number for debugging)
Public DOT As String                ' Substitute for inability to define Const with function call
Public DebugMode As Boolean         ' /DEBUG for debug tracing within all functions (including app-hang handler)

Sub Main()  ' Program starts using Sub Main() (Check/Set in Project->Properties)

    Const Foreground As Long = vbWhite  ' Menu foreground text
    Const Background As Long = vbBlue   ' Menu background text
    
    Dim Browse As Boolean               ' Set to manually browse for a single file
    Dim ConvertJPEG As Boolean          ' Flag to convert JPEG files and call Form1
    Dim CycleMode As Boolean            ' /CycleMode wallpapers
    Dim LaunchWebsite As Boolean        ' /W switch - goto website
    Dim RandomOrder As Boolean          ' /RANDOM or /NORANDOM command-line flag
    Dim ShowHelp As Boolean             ' User chose /H or /? or -H or -?
    Dim Verbose As Boolean              ' All screen and msgbox output
    Dim WildcardSearch As Boolean       ' Set if no filename is given and we default to CWD\*.*
    
    Dim Argc As Integer                 ' Count of cmdline args
    Dim i As Integer                    ' The usual loop index
    Dim Back As Integer                 ' Background menu colour
    Dim Continue As Integer             ' Loop control for wildcard search
    Dim DelayTime As Integer            ' /TIME:x
    Dim DisplayMode As Integer          ' Wallpaper display mode 1:Normal, 2:tiled, 4:stretch
    Dim Fore As Integer                 ' Foreground menu colour
    Dim k As Integer                    ' Inkey() return (ASCII)
    Dim ParamCount As Integer           ' Count of the number of parameters in argv()
    Dim Running As Integer              ' Outer loop control for repeating one folder over
    
    Dim l As Long                       ' Big loop counter for iterating thru' returned pix
    Dim FileCount As Long               ' How many files were found by the directory search
    Dim LastShown As Long               ' Last image shown (/RANDOM mode)
    Dim PicsDisplayed As Long           ' Count of number of wildcard files processed
    Dim Size As Long                    ' Size of the applied or converted BMP file
    
    Dim FileName As String              ' Name built-up from wildcard search
    Dim INfile As String                ' Singular input file
    Dim HomeDir As String               ' The user's HOME directory
    Dim TempDir As String               ' Alternative if HOME directory can't be located
    Dim PathName As String              ' Passed cmdline path/filename argument
    Dim s As String                     ' Debug
    Dim Temp As String                  ' GP temp var
    Dim WorkingPath As String           ' Current path (CurDir())

    Dim Argv As Variant                 ' Array of cmdline args (Variant)
    Dim V As Variant                    ' Used to hold the return from ListFiles()
    
    On Error GoTo LibError              ' Initial error-handling goes to then "no console" error-handler
                                        ' Once DLL availability has been verified change to "On Error GOTO Error"
    
    ' Initialise variables ///////////////////////////////////////////////////////////
    
    'If App.PrevInstance Then AppActivate App.PrevInstance   ' Prevent more than one instance using this
    
    Argc = 0                                    ' Count of Command$ args
    Fore = FOREGROUND_WHITE                     ' Menu foreground colour
    Back = BACKGROUND_BLUE                      ' Menu background colour
    DOT = Chr$(254) & " "                       ' Display blob
    INfile = ""                                 ' Input argument
    PathName = ""                               ' Cmd-line passed path or filename
    ShowHelp = False                            ' /? /H was sent
    Verbose = True                              ' Verbose mode is on by default
    Browse = False                              ' Manual browse-to-folder
    ParamCount = 0                              ' Count of the # of non-switch arguments
    ConvertJPEG = False                         ' Only set if we are given a JPEG file
    WildcardSearch = False                      ' No wildcard search by default
    DelayTime = 60                              ' Default delay time (in seconds) where used
    DisplayMode = 1                             ' Normal display mode (centred)
    FileCount = 0&                              ' Wildcard file-count
    PicsDisplayed = 0&                          ' How many pictures we have displayed (running count)
    DebugMode = False                           ' Debugging OFF
    RandomOrder = True                          ' Default is to display wallpapers in random-order
    LaunchWebsite = False                       ' Only with /W switch
    
    ' Process all args ////////////////////////////////////////////////////////////////////////////////
    
    ERL = 10
    Argc = GetArgs(Command$, Argv)                      ' Get the command line tokenised
    ERL = 20
    For i = 1 To Argc                                   ' Process all command-line arguments
        If Left$(Argv(i), 1) = "-" Then Mid$(Argv(i), 1, 1) = "/" ' Change Unix to DOS args
        If Left$(Argv(i), 1) <> "/" Then                ' Not a switch
            INfile = Argv(i)                            ' Store input filename
            ParamCount = ParamCount + 1                 ' Increment the parameter count (not used yet)
        End If
        
        Argv(i) = UCase$(Argv(i))                       ' Convert to upper case
        ERL = 30
        If Argv(i) = "/B" Then                          ' Browse mode (not implemented yet)
            Browse = True                               ' DLL FileOpen etc. ready
        ElseIf Argv(i) = "/CYCLE" Then                  ' CycleMode thru' all files
            CycleMode = True
        ElseIf Argv(i) = "/DEBUG" Then                  ' Enable debug tracing
            DebugMode = True
        ElseIf Argv(i) = "/H" Or Argv(i) = "/?" Then    ' Help, /H, /?
            ShowHelp = True
        ElseIf Argv(i) = "/NOCYCLE" Then                  ' CycleMode thru' all files
            CycleMode = False
        ElseIf Argv(i) = "/NODEBUG" Then               ' Don't display debug info (override batch file value)
            DebugMode = False
        ElseIf Argv(i) = "/NORANDOM" Then               ' Don't display in random-order
            RandomOrder = False
        ElseIf Argv(i) = "/Q" Then                          ' Verbose mode OFF (Quiet)
            Verbose = False                             ' Turn messages off
        ElseIf Argv(i) = "/STRETCH" Then                ' Display mode is "stretched"
            DisplayMode = 4
        ElseIf Argv(i) = "/TILE" Then                   ' Display mode is "tiled"
            DisplayMode = 2
        ElseIf Left$(Argv(i), 5) = "/TIME" Then         ' /TIME:x or /TIME=x
            ERL = 35
            If Len(Argv(i)) > 6 Then
                Temp = Mid$(Argv(i), 7)
                DelayTime = Max(5, Val(Temp))           ' Minimum delay of 5 seconds per picture
            End If
        ElseIf Argv(i) = "/W" Then                      ' Goto Website
            LaunchWebsite = True
        ElseIf Left$(Argv(i), 1) <> "/" Then            ' Grab the first non-switch argument as our path
            If PathName = "" Then PathName = Argv(i)    ' Store our path argument if not yet set
            Debug.Print "Path name is ["; PathName; "]"
        End If
        
    Next
    If Argc < 1 Then ShowHelp = True    ' No arguments (we MUST exit)
    
    ' End processing args ////////////////////////////////////////////////////////////////////////////////
    
    ERL = 100
    If Not IsVBToolboxDLLInstalled() Then    ' We can't open a console as there is no DLL
        ' But if running silently we just want to quit (or log the error)
        ' (Implies we need to check the args before doing this)
        If Verbose Then MsgBox "Required DLL - " & DLLNAME & " was not found - please reinstall", vbCritical, "Missing DLL"
        ' LogError errorlog, "Missing DLL"  ' TODO LogError() funct
        End
    Else    ' DLL is installed but is it the right version ?
        If LibVersion() < LIBNEEDED And Verbose Then
            MsgBox "Wrong version of " & DLLNAME & " - you need version " & Format$(LIBNEEDED / 100) & " or higher "
            End
        End If
    End If
    On Error GoTo Error             ' Redirect error-handler to main error handler
    
    ' Open the Console /////////////////////////////////////////////////////////////////
    ' We have the DLL available - so now try to open a console window
    ' Do we still want to open a console if not verbose? (run windowless mode?)
    ERL = 200
    If OpenConsole() = 0& And Verbose Then    ' If failed and not silent mode
        MsgBox "Could not open a console window", vbCritical, "Error"
        End
    End If
    
    ' Launch website
    If LaunchWebsite Then
        ShellRun WEBSITE
        End
    End If

    ' Show user-help screen ////////////////////////////////////////////////////////////
    ERL = 300
    If ShowHelp Then 'If help then announce help
        Cls
        ShowBanner Fore, Back
        WriteLn "Sets the Windows wallpaper either singly or every few seconds at random"
        WriteLn "Bitmap and JPEG files may be used - files are converted to BMP before use"
        WriteLn "Converted JPEG files are saved as bitmaps in the user's home directory"
        WriteLn "If a path or wildcard is specified multiple files may be selected"
        WriteLn "Default display-mode is centred but you can use /TILE or /STRETCH"
        WriteLn
        WriteLn "Syntax: " & EXENAME & " <filename|path> [switches]"
        WriteLn
        '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        WriteLn DOT & "/CYCLE     Cycle through all images in the given folder"
        WriteLn DOT & "/NORANDOM  Don't set the wallpaper randomly. Display in unsorted order"
        WriteLn DOT & "/Q         Quiet mode (press any key on the console window to quit)"
        WriteLn DOT & "/TIME:x    Set timer interval to x seconds (current is " & Format$(DelayTime) & ")"
        WriteLn DOT & "/TILE      Tile the wallpaper (centre is the default)"
        WriteLn DOT & "/STRETCH   Stretch the wallpaper to fit the screen (doesn't keep aspect ratio)"
        WriteLn DOT & "/W         Launch/goto the VBToolbox website"
                
        'WriteLn DOT & "/B       Browse to open a file"
        WriteLn
        WriteLn "Needs: MSLIB145.DLL Website: " & WEBSITE
        WriteLn
        If Not InNativeConsole() Then    ' Will not work with CGI apps
            WriteLn "This program may be converted to a native console application"
            WriteLn "Use (v5.12.8078) EDITBIN.EXE /EDIT /SUBSYSTEM:CONSOLE <this-file>"
            WriteLn "Download with freeware MASM32 assembler package - http://www.masm32.com"
            WriteLn
            Pause
        End If
        
        CloseConsole    ' Close our console and quit
        End
    End If
    '///////////////////////////////////////////////////////////////////////////////////
    
    'Build and check the input filename ////////////////////////////////////////////////
    ERL = 350
    PathName = GetNormalisedPath(PathName)  ' Ensure path is normalised/canonicalised
    '"." etc. should not occur
    If PathName = "" Or PathName = "." Or PathName = ".\" Then            ' No input file arg was given so look in CWD
        WorkingPath = CurDir()          ' Get the current directory
        INfile = WorkingPath
        INfile = AddTrailingSlash(INfile)
        INfile = INfile & "*.*"  ' We look for BMP and JPEG files
        WildcardSearch = True
        If DebugMode Then Debug.Print "DEBUG: No path was given =["; INfile; "]"
    Else
        INfile = PathName           ' Collect whatever argument was given
        'INfile = AddTrailingSlash(INfile)
        WorkingPath = INfile
    End If
    
    ERL = 380                       ' This block could be incorporated with that above
    If InStr(INfile, "*") > 0 Or InStr(INfile, "?") > 0 Then    ' This is already *.* etc.
        WildcardSearch = True
        WorkingPath = FileNameToPath(INfile)
    ElseIf DirExists(INfile) Then       ' INfile is a drectory not a filename
        WorkingPath = INfile
        INfile = AddTrailingSlash(INfile)
        INfile = INfile & "*.*"         ' Append wildcard search mask
        WildcardSearch = True           ' Set flag
    End If
    
    WorkingPath = AddTrailingSlash(WorkingPath)
            
    ERL = 400
    ' We will store any converted JPEG files to the user profile home dir
    HomeDir = Environ("USERPROFILE")    ' Check this later - may convert to API
    HomeDir = AddTrailingSlash(HomeDir) ' Canonicalise
    
    If DirExists(HomeDir) = False Then  ' Can't find the user's home-directory
        TempDir = Environ("TEMP")       ' Try to save our converted files (if any) to TEMP
        If DirExists(TempDir) Then
            HomeDir = TempDir
        Else
            HomeDir = CurDir()          ' Save to the current directory as a last-try
        End If
    Else    ' Exists: But try to put the temporary BMP files somewhere more sensible if possible
        If DirExists(HomeDir & "My Documents\My Pictures\") Then
            HomeDir = HomeDir & "My Documents\My Pictures\"           ' Expand HomeDir
        ElseIf DirExists(HomeDir & "Local Settings\Temp\") Then
            HomeDir = HomeDir & "Local Settings\Temp\"              ' Expand HomeDir
        End If
    End If
    
    If DebugMode Then
        WriteLn "DEBUG: Processing input path  [" & INfile & "]"
        WriteLn "DEBUG: Processing output path [" & HomeDir & "]"
    End If
    
    ' /// End processing filenames /////////////////////////////////////////////////////
    
    ' /// Load and convert the picture /////////////////////////////////////////////////
    
    ERL = 500
    If Verbose Then ShowBanner Fore, Back           ' Display our nice banner
    
    If DirExists(WorkingPath) = False Then             ' Check to see if our normalised folder exists
        If Verbose Then WriteLn DOT & "Can't find folder " & Chr$(34) & WorkingPath & Chr$(34)
        If Not InNativeConsole Then Pause
        CloseConsole
        End         ' Exit the program (verbose nor not)
    End If
    
    ' ANNOUNCE ///////////////////////////////////////////////////////
    ERL = 600
    If Verbose Then
        SetConsoleAttributes FOREGROUND_LIGHTBLUE, BACKGROUND_BLACK
        WriteLn DOT & "Using VBToolbox DLL - (" & DLLNAME & ") - v" & DLLVersion()
        WriteLn DOT & "Website: " & WEBSITE
        ClearConsoleAttributes
        WriteLn DOT & "(Process ID: " & Format$(GetPID) & ")"
        
        Writes DOT & "Displaying pictures in "     ' NO CRLF!
        If RandomOrder Then
            Writes "random order"
        Else
            Writes "linear (unsorted) folder order"
        End If
        WriteLn " use /NORANDOM to change"
        If DisplayMode = 4 Then
            WriteLn DOT & "Stretching wallpaper to fit"
        ElseIf DisplayMode = 2 Then
            WriteLn DOT & "Tiling wallpaper - use /CENTRE or /STRETCH to change"
        Else
            WriteLn DOT & "Centring with no stretching - use /TILE or /STRETCH to change"
        End If
        If CycleMode Then WriteLn DOT & "Changing every " & Format$(DelayTime) & " seconds - use /TIME:x to change"
        SetConsoleAttributes FOREGROUND_LIGHTRED, BACKGROUND_BLACK
        WriteLn DOT & "If SetWall hangs converting a JPEG then download Bad-Peggy JPEG validator"
        WriteLn "  from http://www.hotpixel.net (Free) and scan your images for errors"
        ClearConsoleAttributes
    End If
        
    ERL = 700
    If WildcardSearch Then                                  ' Scan multiple files mode
        If DebugMode Then WriteLn "DEBUG: Searching [ " & INfile & "]"
        If Verbose Then WriteLn DOT & "Searching " & WorkingPath & " for JPG and BMP files ..."
        V = ListFiles(INfile, "*.jpeg, *.jpg, *.bmp")       ' Get String() as Variant
        If DebugMode Then WriteLn "DEBUG: Search complete."
        
        If Not IsEmpty(V) Then                              ' We found something
            ERL = 800
            FileCount = UBound(V) - LBound(V) + 1             ' Work out the count dependent on array-base
            If Verbose Then WriteLn DOT & "Found " & Format$(FileCount) & " file(s) ..."
            
            PicsDisplayed = 0&
            If RandomOrder Then
                ERL = 900
                Randomise                                   ' Set the RNG
                Continue = True
                If DebugMode Then WriteLn "DEBUG: Starting even loop..."
                LastShown = 0                               ' Set duplicate image-tracker to 0
                Do While Continue                           ' If CycleMode or Do forever (or until user quits)
                    l = 0
                    Do
                        l = Random(LBound(V), UBound(V))
                        If DebugMode Then WriteLn "DEBUG: Selecting item " & Format$(l)
                        DoEvents                            ' We are unlikely to get stuck in a loop, but...
                    Loop While l = LastShown And FileCount > 1 ' Keep fetching until numbers differ
                    ' (If we have one file only then we would be stuck in an loop here!)
                    ' Announce the first wallpaper transform only... (since this may take time on VMs)
                    If PicsDisplayed = 0 And Verbose Then WriteLn DOT & "Setting wallpaper to: " & Chr$(34) & V(l) & Chr$(34)
                    
                    FileName = WorkingPath & V(l)
                    If DebugMode Then WriteLn "DEBUG: Calling ConvertAndSetWallpaper ..."
                    Size = ConvertAndSetWallpaper(FileName, HomeDir, Verbose, DisplayMode) 'Change it
                    If Size > 0& Then 'We set a file
                        PicsDisplayed = PicsDisplayed + 1               ' We found (but may not have set) x files
                        If Verbose Then WriteLn DOT & Format$(Now, "hh:mm:ss") & " - Wallpaper: " & Chr$(34) & V(l) & Chr$(34) & " (" & Comma(Size) & " bytes)"
                        If Not CycleMode Then
                            If Verbose Then WriteLn DOT & "Use /CYCLE to repeat continuously"
                            Exit Do                                 ' We only continue if args include /CycleMode (below)
                        Else                                        ' SLEEP as required
                            If Verbose Then DisplayExitReminder PicsDisplayed, DelayTime      ' Reassure the user they can quit if they wish
                            If Not SleepAndWaitForKeyboard(DelayTime, Verbose) Then ' Check for keyboard exit
                                Exit Do                                 ' A key was pressed so quit the loop
                            End If
                        End If
                    End If
                    LastShown = l                                           ' Track this image (prevent repetition)
                Loop
                
            Else                                                            ' Display pictures serially
            
                ERL = 950
                Continue = True
                Do While Continue                                           ' Loop forever (or until key pressed)
                    For l = LBound(V) To UBound(V)                          ' For all wallpapers found ...
                        FileName = WorkingPath & V(l)
                        If DebugMode Then WriteLn "DEBUG: Calling ConvertAndSetWallpaper ..."
                        Size = ConvertAndSetWallpaper(FileName, HomeDir, Verbose, DisplayMode) ' Change it
                        If Size > 0& Then 'We set a file
                            PicsDisplayed = PicsDisplayed + 1 ' We found (but may not have set) x files
                            If Verbose Then WriteLn DOT & Format$(Now, "hh:mm:ss") & " - Wallpaper: " & Chr$(34) & V(l) & Chr$(34) & " (" & Comma(Size) & " bytes)"
                            If Not CycleMode Then
                                If Verbose Then WriteLn DOT & "Use /CYCLE to repeat continuously" ' Single-file mode
                                Continue = False                        ' Set outer loop exit flag
                                Exit For                                ' We only continue if args include /CycleMode (below)
                            Else                                        ' SLEEP as required
                                If Verbose Then DisplayExitReminder PicsDisplayed, DelayTime      ' Reassure the user they can quit if they wish
                                If Not SleepAndWaitForKeyboard(DelayTime, Verbose) Then ' Check for keyboard exit
                                    Exit For                            ' A key was pressed so quit the loop
                                End If
                            End If
                        End If
                    Next
                    ' We displayed the serial list of wallpapers and now we will start over again...
                    If Verbose And Continue Then
                        SetConsoleAttributes FOREGROUND_LIGHTGREEN, BACKGROUND_BLACK
                        WriteLn DOT & "Repeating contents of folder " & INfile
                        ClearConsoleAttributes
                    End If
                Loop
            End If
        Else                                                            ' NO FILES found!
            If Verbose Then
                WriteLn DOT & "No supported image files (JPEG,JPG or BMP) in " & WorkingPath
            End If
        End If
    End If ' End If WildcardSearch Then                                  ' Scan multiple files mode
    ' End load and convert picture //////////////////////////////////////////////
    
    ' Clean up and exit /////////////////////////////////////////////////////////
    ERL = 1000
    CloseConsole        ' Close console always
    End
    Exit Sub

LibError:           ' When checking to see if the DLL exists our error-handler can't call any lib functions
        Debug.Print "LibError "; Err.Number; " ("; Err.Description; ") in Main() at line " & Format$(ERL)
        App.logevent "Error: " & Err.Number & " ( " & Err.Description & " ) in main() at line " & Format$(ERL), vbLogEventTypeError
        Resume Next ' So we just resume
Error:              ' General Error-Handling routine ////////////////////////////////////
                    ' This handler uses the DLL console functions - we could have one handler and set
                    ' a flag according to whether DLL installed or not
    Debug.Print "Error "; Err.Number; " ("; Err.Description; ") in Main() at line " & Format$(ERL)
    If Verbose Then
        If ConsoleOpen() Then
            WriteLn "Error: " & Err.Number & " ( " & Err.Description & " ) in main() at line " & Format$(ERL)
        End If
    Else
        App.logevent "Error: " & Err.Number & " ( " & Err.Description & " ) in main() at line " & Format$(ERL), vbLogEventTypeError
    End If
    Resume Next
End Sub
Public Function FileNameToPath(FileName As String) As String
    'TODO: Convert to DLL function
    Dim p As Long
    On Error GoTo Error
    FileNameToPath = ""
    If FileName <> "" Then
        p = InStrRev(FileName, "\")
        If p > 0 Then
            FileNameToPath = Left$(FileName, p)
        Else
            FileNameToPath = FileName
        End If
    End If
    Exit Function
    
Error:
    ' No specific handler
    Resume Next
End Function
Public Function SleepAndWaitForKeyboard(ByVal SleepTimeSeconds As Integer, Verbose As Boolean) As Boolean
    ' Returns "False" if we were interrupted by keypress and failed to complete the "sleep"
    ' Returns "True" if we successfully slept throughout the whole requested period
    ' We return "True" for [SPACE] but quit here so we can use this to advance to the next image
    
    Dim i As Integer
    Dim k As Integer
    On Error GoTo Error                     ' Trivial funct should not get errors ;)
    
    SleepAndWaitForKeyboard = True          ' Preset to success
    For i = 1 To SleepTimeSeconds * 10      ' && SLEEP responsively
        WinSleep 100                        ' Keep CPU nice (sleep for 1 / 10 th of a second)
        DoEvents                            ' Stay even more responsive
        k = InKey()                         ' Check the keyboard for input

        If k > 0 Then             ' A key event was found (and was n
            'If Verbose Then WriteLn "DEBUG: Key value was " & Format$(k)
            If k = 32 Then
                If Verbose Then                 ' Alert the careless-fingered user
                    SetConsoleAttributes FOREGROUND_LIGHTYELLOW, BACKGROUND_BLACK
                    WriteLn DOT & "Reading next image ..."
                    ClearConsoleAttributes
                End If

                SleepAndWaitForKeyboard = True ' A SPACE key was pressed (fake true return)
            Else
                If Verbose Then                 ' Alert the careless-fingered user
                    SetConsoleAttributes FOREGROUND_LIGHTRED, BACKGROUND_BLACK
                    WriteLn DOT & "Stopped."
                    ClearConsoleAttributes
                End If
                SleepAndWaitForKeyboard = False ' Some other key was pressed
            End If
            Exit For                            ' Bail out of the for(;;) loop
        End If
    Next
    Exit Function
    
Error:
    ' No specific handler
    Resume Next
End Function
Public Sub DisplayExitReminder(ByVal FileCount As Long, ByVal DelayTime As Integer)
    'Periodically reassure the user who has not RTFM that they can exit at any time
    If FileCount = 1 Or FileCount Mod 10 = 0 Then ' Display alert every 5 changes
            SetConsoleAttributes FOREGROUND_LIGHTYELLOW, BACKGROUND_BLACK
            WriteLn DOT & "Changing every " & Format$(DelayTime) & " second(s) - SPACE for next image - any other key to quit"
            ClearConsoleAttributes
    End If
End Sub
Public Sub ShowBanner(Fore As Integer, Back As Integer)
    ' Display our banner
    SetConsoleAttributes Fore, Back
    Writes " " & EXENAME & " v" & VERSION & " - Copyright (C) " & AUTHOR & " - " & VERDATE & " (Freeware)      "
    ClearConsoleAttributes
    WriteLn vbCrLf ' CRLF x2
End Sub
Public Function ConvertAndSetWallpaper(ByVal FileName As String, ByVal OutputDir As String, _
    ByVal Verbose As Boolean, DisplayMode As Integer) As Long
    
    ' This is where we check the filename/path and convert file as required
    ' FileName must be a FQ path+filename or file will NOT be found
    ' Global objects: Form1, Timer1
    ' Sets a wallpaper - if it is a JPEG it will be converted and saved to the user's home dir
    ' Note that due to a bug in the VB Picture/Image control some JPEGs will cause this to hang with no error msg
    ' This is a pain as it will run the CPU 100% and fail to report any error
    ' Note that when running in QEMU it may take a long time to load/convert JPEG files
    ' It is not abnormal for an average-sized BMP conversion of a JPEG to be around 10Mb in size
    ' The return value could be altered to return the size of the displayed BMP file
    
    Dim Typ As String           ' Used to extract the filetype part FNSplit
    Dim Name As String          ' Used to extract only the name part FNSplit
    Dim BMPFile As String       ' Our output BMP filename for converted JPEG files
    Dim p As Long
    Dim i As Integer
    On Error GoTo Error

    ConvertAndSetWallpaper = 0&                             ' Preset to FAILED return
    
    If FileName <> "" And OutputDir <> "" Then              ' Sanity-check for no filename and O/P path
        If FileExists(FileName) Then                        ' If our file (of whatever kind) exists
            ' We must assume that another process may have deleted the file who's name is supplied
            OutputDir = AddTrailingSlash(OutputDir)         ' Ensure we put files where we intend them to go!
            
            Typ = FileType(FileName)
            If Typ <> "" Then                               ' We have a dot in the filename
                Typ = LCase$(Typ)                           ' Get the filetype
                If Typ = "bmp" Or Typ = "jpg" Or Typ = "jpeg" Then   ' We only handle these types
                
                    If Typ = "bmp" Then                ' Just set the picture "as is"
                        SetWallpaper FileName, DisplayMode
                        ConvertAndSetWallpaper = FileLen(FileName)       ' Set return status
                    Else                                    ' Else we need to convert the JPEG file
                        p = InStrRev(FileName, "\")
                        If p > 0 Then Name = Mid$(FileName, p + 1) Else Name = FileName
                        p = InStrRev(Name, ".")
                        If p > 0 Then Name = Left$(Name, p - 1) ' Should never fail since we tested for "."
                        'Uncomment to use the same name over and again each time
                        BMPFile = OutputDir & EXENAME & "-Wallpaper.bmp"       ' Build the filename
                        'Uncomment to keep original name
                        'BMPFile = OutputDir & Name & ".bmp"
                        
If DebugMode Then
    WriteLn "DEBUG: Converting and saving [" & FileName & "]"
    WriteLn "       As output BMP [" & BMPFile & "]"
End If
                        If JPEGCheck(FileName) Then         ' v1.01+ Basic sanity-checks on the file's header(s)
                            With Form1                      ' Reference the hidden form's controls
                                On Error GoTo ImageError
                                Set .Image1.Picture = LoadPicture
                                Set .Image1.Picture = Nothing
                                .Timer1.Interval = 10000    ' 10,000 msecs (10 seconds) to convert it
                                .Timer1.Enabled = True      ' Timer will NOT act if the process locks-up
If DebugMode Then
    WriteLn "DEBUG: Loading JPEG image " & FileName & " into a Picture object"
End If
                                'LoadPicture FileName       ' Alternative
                                Set .Image1.Picture = LoadPicture(FileName) 'ERR: 481 invalid picture??
                                For i = 1 To 5              ' Wait a few milliseconds
                                    WinSleep 10
                                    DoEvents
                                Next
If DebugMode Then
    WriteLn "DEBUG: Loaded JPEG " & FileName & " OK"
    WriteLn "DEBUG: Saving as " & BMPFile
End If
                                On Error GoTo ImageError
                                SavePicture .Image1, BMPFile
                        
If DebugMode Then
    WriteLn "DEBUG: Saved converted JPEG as " & BMPFile & " OK"
End If
                                On Error GoTo Error
                            End With
                            If ImageERR = 0 Then
                                If FileExists(BMPFile) Then ' Check converted return was OK
If DebugMode Then
    WriteLn "DEBUG: Asking Win32 API to set wallpaper to: " & BMPFile & " OK"
End If
                                    If SetWallpaper(BMPFile, DisplayMode) Then
                                        ConvertAndSetWallpaper = FileLen(BMPFile)  ' Set return status
                                    End If
                                End If
                            End If
                            ImageERR = 0                    ' Reset ImageERR always
                        Else                                ' JPEGCheck - image is corrupted
                            If Verbose Then
                                WriteLn "JPEG image [" & FileName & "] appears to be corrupted"
                            End If
                        End If                              ' JPEGCheck() passed OK
                    End If
                End If
            End If
        Else
            WinSleep 100    ' If looping can run CPU 100% preventing desktop user-interaction
        End If
    End If
    
Fail:
    Form1.Timer1.Enabled = False                            ' Disable the crash-handler
    Exit Function
    
ImageError:     ' We call here only if an error occurred during image conversion
    Set Form1.Image.Picture = Nothing
    ImageERR = Err.Number
    If Verbose Then
        WriteLn "Graphics Error: " & Format$(ImageERR) & " (" & Err.Description & ") occurred during image conversion"
    End If
    Resume Fail
    
Error:      ' General error-handling
    Debug.Print "*Error "; Err.Number; " ("; Err.Description; ") in Main() at line " & Format$(ERL)
    If Verbose Then
        If ConsoleOpen() Then
            WriteLn "Error: " & Err.Number & " ( " & Err.Description & " ) in main() at line " & Format$(ERL)
        End If
    End If
    Resume Next
End Function
' // EOF /////////////////////////////////////////////////////////////////////////
